'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports['default'] = createCRUD;

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj['default'] = obj; return newObj; } }

var _immutable = require('immutable');

var _actions = require('../actions');

var _utils = require('../utils');

var utils = _interopRequireWildcard(_utils);

var _constants = require('../constants');

var INITIAL_STATE = (0, _immutable.Map)({
  folders: (0, _immutable.Map)(),
  documents: (0, _immutable.Map)()
});

exports.INITIAL_STATE = INITIAL_STATE;
/**
 * Creates CRUD action types, actions and reducer for PouchDB database.
 *
 * @param {PouchDB} db - PouchDB instance
 * @param {string} [mountPoint] - prefix to use for action types
 * @param {string} [prefix=mountPoint] - prefix to use for action types
 * @param {Object} [opts={}] options
 *
 * @returns {Object} contains `actionTypes`, `actions`, `reducer`
 */

function createCRUD(db, mountPoint) {
  var prefix = arguments.length <= 2 || arguments[2] === undefined ? null : arguments[2];
  var opts = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];

  prefix = prefix || mountPoint;
  var urlPrefix = '/' + prefix;

  var defaultOpts = opts;

  var actionTypes = {};
  for (var action in _constants.ACTIONS) {
    actionTypes[action] = {};
    for (var type in _constants.TYPES) {
      actionTypes[action][type] = utils.createActionType(prefix, action, type);
    };
  };

  var allDocs = function allDocs(folder, params) {
    if (folder === undefined) folder = '';

    var mergedParams = _extends({
      attachments: true,
      include_docs: true
    }, params);
    return (0, _actions.createPromiseAction)(function () {
      return db.allDocs(mergedParams);
    }, actionTypes.allDocs, { folder: folder });
  };

  var query = function query(fun, folder, params) {
    if (folder === undefined) folder = '';

    var mergedParams = _extends({
      attachments: true,
      include_docs: true
    }, params);
    return (0, _actions.createPromiseAction)(function () {
      return db.query(fun, mergedParams);
    }, actionTypes.query, { folder: folder });
  };

  var get = function get(docId) {
    var params = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

    return (0, _actions.createPromiseAction)(function () {
      return db.get(docId, params);
    }, actionTypes.get, { docId: docId });
  };

  var put = function put(doc, params) {
    return (0, _actions.createPromiseAction)(function () {
      return db.put(doc, params);
    }, actionTypes.put, { doc: doc });
  };

  var remove = function remove(doc, params) {
    return (0, _actions.createPromiseAction)(function () {
      return db.remove(doc, params);
    }, actionTypes.remove, { doc: doc });
  };

  var actions = {
    allDocs: allDocs,
    query: query,
    get: get,
    put: put,
    remove: remove
  };

  var reducer = function reducer(state, action) {
    if (state === undefined) state = INITIAL_STATE;

    switch (action.type) {
      case actionTypes.query.success:
      case actionTypes.allDocs.success:
        var ids = (0, _immutable.List)(action.payload.rows.map(function (row) {
          return row.id;
        }));
        var documents = (0, _immutable.Map)(action.payload.rows.map(function (row) {
          return [row.id, (0, _immutable.fromJS)(row.doc)];
        }));
        return state.setIn(['folders', action.folder], ids).mergeIn(['documents'], documents);
      case actionTypes.put.success:
        return state.setIn(['documents', action.payload.id], (0, _immutable.fromJS)(_extends({}, action.doc, {
          _rev: action.payload.rev
        })));
      case actionTypes.get.success:
        return state.setIn(['documents', action.payload._id], (0, _immutable.fromJS)(action.payload));
      case actionTypes.remove.success:
        var newState = state.deleteIn(['documents', action.payload.id]);
        newState.get('folders').map(function (list, k) {
          var index = list.indexOf(action.payload.id);
          if (index != -1) {
            newState = newState.setIn(['folders', k], list['delete'](index));
          }
        });
        return newState;
      default:
        return state;
    }
  };

  var paths = {
    list: urlPrefix + '/',
    detail: utils.createDetailLink(urlPrefix, ':id'),
    edit: urlPrefix + '/:id/edit/',
    create: urlPrefix + '/new/'
  };

  return {
    actions: actions,
    actionTypes: actionTypes,
    reducer: reducer,
    mountPoint: mountPoint,
    paths: paths,
    urlPrefix: urlPrefix,
    db: db
  };
}